﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

/************************ 用法start ************************* /

//1.声明一个继承RandomObject的实体类，如：*
public class AdvertEntity : RandomObject
{
    /// <summary>
    /// 名称
    /// </summary>
    public string Name { set; get; }
    /// <summary>
    /// 描述
    /// </summary>
    public string Description { set; get; }
    //...其它相关的字段/属性
}

//2.初始化调用数据，如：
    //初始化模拟权重数据
    List<Advert> list = new List<Advert>();
    list.Add(new Advert { Name = "A", Weight = 1 });
    list.Add(new Advert { Name = "B", Weight = 2 });
    list.Add(new Advert { Name = "C", Weight = 2 });
    list.Add(new Advert { Name = "D", Weight = 3 });
    list.Add(new Advert { Name = "E", Weight = 4 });
    list.Add(new Advert { Name = "F", Weight = 5 });
    list.Add(new Advert { Name = "G", Weight = 60 });
    //根据权重随机取指定记录条数
    List<Advert> newList = RandomHelper.GetRandomList<Advert>(list, 2);  //这个就是用法，newList就是随机取出的记录（第二个参数就是随机取多少条）

/ ************************ 用法end *************************/


/// <summary>
/// 描述： 根据权重随机抽取指定条数的记录类库
/// 作者:  Foolin
/// 邮箱： Foolin@126.com
/// 日期:  2012年3月22日
/// 微博： http://weibo.com/foolin
/// 博客:  http://foolin.cnblogs.com
/// 版权： 免费、开源，唯一有个小小要求，就是保留原作者信息。
/// </summary>
public class RandomHelper
{

    /// <summary>
    /// 算法：
    /// 1.每个广告项权重+1命名为w，防止为0情况。
    /// 2.计算出总权重n。
    /// 3.每个广告项权重w加上从0到(n-1)的一个随机数（即总权重以内的随机数），得到新的权重排序值s。
    /// 4.根据得到新的权重排序值s进行排序，取前面s最大几个。
    /// </summary>
    /// <param name="list">原始列表</param>
    /// <param name="count">随机抽取条数</param>
    /// <returns></returns>
    public static List<T> GetRandomList<T>(List<T> list, int count) where T: RandomObject
    {
        if (list == null || list.Count <= count || count <= 0)
        {
            return list;
        }

        //计算权重总和
        int totalWeights = 0;
        for (int i = 0; i < list.Count; i++)
        {
            totalWeights += list[i].Weight + 1;  //权重+1，防止为0情况。
        }

        //随机赋值权重
        Random ran = new Random(GetRandomSeed());  //GetRandomSeed()防止快速频繁调用导致随机一样的问题 
        List<KeyValuePair<int, int>> wlist = new List<KeyValuePair<int, int>>();    //第一个int为list下标索引、第一个int为权重排序值
        for (int i = 0; i < list.Count; i++)
        {
            int w = (list[i].Weight + 1) + ran.Next(0, totalWeights);   // （权重+1） + 从0到（总权重-1）的随机数
            wlist.Add(new KeyValuePair<int, int>(i, w));
        }

        //排序
        wlist.Sort(
          delegate(KeyValuePair<int, int> kvp1, KeyValuePair<int, int> kvp2)
          {
              return kvp2.Value - kvp1.Value;
          });

        //根据实际情况取排在最前面的几个
        List<T> newList = new List<T>();
        for (int i = 0; i < count; i++)
        {
            T entiy = list[wlist[i].Key];
            newList.Add(entiy);
        }

        //随机法则
        return newList;
    }


    /// <summary>
    /// 随机种子值
    /// </summary>
    /// <returns></returns>
    private static int GetRandomSeed()
    {
        byte[] bytes = new byte[4];
        System.Security.Cryptography.RNGCryptoServiceProvider rng = new System.Security.Cryptography.RNGCryptoServiceProvider();
        rng.GetBytes(bytes);
        return BitConverter.ToInt32(bytes, 0);
    }


}



/// <summary>
/// 权重对象
/// </summary>
public class RandomObject
{
    /// <summary>
    /// 权重
    /// </summary>
    public int Weight { set; get; }
}